#ifndef MODPERL_CONSTANTS_H
#define MODPERL_CONSTANTS_H

/*
 * *********** WARNING **************
 * This file generated by ModPerl::Code/0.01
 * Any changes made here will be lost
 * ***********************************
 * 01: lib/ModPerl/Code.pm:740
 * 02: lib/ModPerl/Code.pm:766
 * 03: Makefile.PL:391
 * 04: Makefile.PL:102
 */

#define MP_ENOCONST -3

SV *modperl_constants_lookup_apr_const(pTHX_ const char *name);
SV *modperl_constants_lookup_apache2_const(pTHX_ const char *name);
SV *modperl_constants_lookup_modperl(pTHX_ const char *name);
const char **modperl_constants_group_lookup_apr_const(const char *name);
const char **modperl_constants_group_lookup_apache2_const(const char *name);
const char **modperl_constants_group_lookup_modperl(const char *name);

#endif /* MODPERL_CONSTANTS_H */
